#ifndef MAIL_STORAGE_H

#define MAIL_STORAGE_H


#include "maildriver.h"

#include <pthread.h>
#include <inttypes.h>

struct mail_storage;

struct mail_storage_driver {
  char * name;
  int (* connect)(struct mail_storage * storage);
  int (* get_folder)(struct mail_storage * storage,
      char * pathname, mailsession ** result);
  void (* free_data)(void * data);
};

typedef struct mail_storage_driver mail_storage_driver;

struct mail_storage {
  char * id;
  void * data;
  mailsession * session;
  mail_storage_driver * driver;
  clist * shared_folders;
};

enum {
  CONNECTION_TYPE_PLAIN,
  CONNECTION_TYPE_STARTTLS,
  CONNECTION_TYPE_TRY_STARTTLS,
  CONNECTION_TYPE_TLS,
};

struct mbox_mail_storage {
  char * pathname;
  
  int cached;
  char * cache_directory;
};

struct mh_mail_storage {
  char * pathname;
  
  int cached;
  char * cache_directory;
};

enum {
  IMAP_AUTH_TYPE_PLAIN,
  IMAP_AUTH_TYPE_SASL_ANONYMOUS,
  IMAP_AUTH_TYPE_SASL_CRAM_MD5,
  IMAP_AUTH_TYPE_SASL_KERBEROS_V4,
  IMAP_AUTH_TYPE_SASL_PLAIN,
  IMAP_AUTH_TYPE_SASL_SCRAM_MD5,
  IMAP_AUTH_TYPE_SASL_GSSAPI,
  IMAP_AUTH_TYPE_SASL_DIGEST_MD5,
};

struct imap_mail_storage {
  char * servername;
  uint16_t port;
  int connection_type;
  
  int auth_type;
  char * login;
  char * password;

  int cached;
  char * cache_directory;
};

enum {
  POP3_AUTH_TYPE_PLAIN,
  POP3_AUTH_TYPE_APOP,
  POP3_AUTH_TYPE_TRY_APOP,
  POP3_AUTH_TYPE_SASL_ANONYMOUS,
  POP3_AUTH_TYPE_SASL_CRAM_MD5,
  POP3_AUTH_TYPE_SASL_KERBEROS_V4,
  POP3_AUTH_TYPE_SASL_PLAIN,
  POP3_AUTH_TYPE_SASL_SCRAM_MD5,
  POP3_AUTH_TYPE_SASL_GSSAPI,
  POP3_AUTH_TYPE_SASL_DIGEST_MD5,
};

struct pop3_mail_storage {
  char * servername;
  uint16_t port;
  int connection_type;

  int auth_type;
  char * login;
  char * password;

  int cached;
  char * cache_directory;
};


enum {
  NNTP_AUTH_TYPE_PLAIN,
};

struct nntp_mail_storage {
  char * servername;
  uint16_t port;
  int connection_type;

  int auth_type;
  char * login;
  char * password;

  int cached;
  char * cache_directory;
};

struct mail_folder {
  char * pathname;
  char * virtual_name;
  
  struct mail_storage * storage;

  mailsession * session;
  int shared_session;
  clistiter * pos;

  struct mail_folder * parent;
  int32_t sibling_index;
  carray * children;
};

struct mail_storage * mail_storage_new(char * id);

void mail_storage_free(struct mail_storage * storage);

int mail_storage_connect(struct mail_storage * storage);

void mail_storage_disconnect(struct mail_storage * storage);

int mail_storage_get_folder(struct mail_storage * storage,
    char * pathname, mailsession ** result);

/* mbox storage */

int mbox_mail_storage_init(struct mail_storage * storage,
			   char * pathname,
			   int cached, char * cache_directory);

/* pop3 storage */

int pop3_mail_storage_init(struct mail_storage * storage,
			   char * servername,
			   uint16_t port,
			   int connection_type,
			   int auth_type,
			   char * login,
			   char * password,
			   int cached,
			   char * cache_directory);

/* imap storage */

int imap_mail_storage_init(struct mail_storage * storage,
			   char * servername,
			   uint16_t port,
			   int connection_type,
			   int auth_type,
			   char * login,
			   char * password,
			   int cached,
			   char * cache_directory);

int nntp_mail_storage_init(struct mail_storage * storage,
			   char * servername,
			   uint16_t port,
			   int connection_type,
			   int auth_type,
			   char * login,
			   char * password,
			   int cached,
			   char * cache_directory);

int mh_mail_storage_init(struct mail_storage * storage,
			 char * pathname,
			 int cached,
			 char * cache_directory);

struct mail_folder * mail_folder_new(struct mail_storage * storage,
    char * pathname, char * virtual_name);

void mail_folder_free(struct mail_folder * folder);

int mail_folder_add_child(struct mail_folder * parent,
        struct mail_folder * child);

int mail_folder_detach_parent(struct mail_folder * folder);

int mail_folder_connect(struct mail_folder * folder);

void mail_folder_disconnect(struct mail_folder * folder);

#endif


